#' Get or set configuration options that control AntBioR behaviour
#'
#' @author Australian Antarctic Data Centre \email{aadc@@aad.gov.au}
#' @references \url{http://www1.data.antarctica.gov.au} \url{http://www.biodiversity.aq/}
#'
#' Invoking \code{antbio_config()} with no arguments returns a list with the current values of the options.
#'
#' @param \dots Options can be defined using name=value. Valid options are:
#' \itemize{
#'   \item reset: \code{antbio_config("reset")} will reset the options to their default values
#'   \item caching string: caching can be
#'     "on" (results will be cached, and any cached results will be re-used),
#'     "refresh" (cached results will be refreshed and the new results stored in the cache), or
#'     "off" (no caching, default).
#'   \item cache_directory string: the directory to use for the cache.
#'     By default this is a temporary directory, which means that results will only be cached
#'     within an R session and cleared automatically when the user exits R. The user may wish to set this to a non-temporary directory for
#'     caching across sessions. The directory must exist on the file system.
#'   \item verbose logical: should AntBioR give verbose output to assist debugging?  (default=FALSE)
#'   \item user_agent string: the user-agent string used with all web requests.
#'     Default = "AntBioR" with version number, R version and date and user platform
#'   \item base_url_aadc string: the base url for AADC applicatons (default="http://www1.data.antarctica.gov.au/")
#'   \item base_url_biodiversityaq string: the base url for biodiversity.aq services (default="http://data.biodiversity.aq")
#' }
#'
#' @return For antbio_config(), a list of all options. When antbio_config(...) is
#' called with arguments, nothing is returned but the configuration is set.
#'
#' @examples
#'
#' antbio_config()
#' antbio_config(caching="off",verbose=TRUE)
#' antbio_config("reset")
#'
#' @export antbio_config
#'
antbio_config=function(...) {
    ## get or set options that control AntBioR behaviour
    ## options are stored as a global option with the name defined in antbio_option_name
    antbio_option_name="AntBioR_config"
    user_options=list(...) ## the options passed by the user

    ## default user-agent string
    version_string="version unknown"
    suppressWarnings(try(version_string<-utils::packageDescription('AntBioR')[["Version"]],silent=TRUE)) ## get the AntBioR version, if we can
    user_agent_string=paste("AntBioR ",version_string," (",R.Version()$version.string,"/",R.Version()$platform,")",sep="")

    ## set default options
    default_options=list(caching="on",cache_directory=tempdir(),user_agent=user_agent_string,download_reason_id=NA,verbose=FALSE,base_url_aadc="https://www1.data.antarctica.gov.au/",base_url_biodiversityaq="http://data.biodiversity.aq/")

    ## define allowed options, for those that have restricted values
    allowed_options=list(caching=c("on","off","refresh"))

    ## has the user asked to reset options to defaults?
    if (identical(tolower(user_options),"reset")) {
        temp=list(default_options)
        names(temp)=antbio_option_name
        options(temp)
    } else if (identical(tolower(user_options),"internal")) {
        ## change AADC URL for use within AAD network
        antbio_config(base_url_aadc="http://www1.data.antarctica.gov.au/")
    } else if (identical(tolower(user_options),"external")) {
        ## change AADC URL for use within AAD network
        antbio_config(base_url_aadc="https://www1.data.antarctica.gov.au/")
    } else if (identical(tolower(user_options),"local")) {
        ## change AADC URL for local-machine testing
        antbio_config(base_url_aadc="http://localhost:8888/")
        ## also need to unset proxy, grrr
        Sys.setenv(http_proxy_server="")
        Sys.setenv(https_proxy_server="")
    } else {
        names(user_options)=tolower(names(user_options))
        ## has the user specified something we don't recognize?
        known_options=names(default_options)
        unknown=setdiff(names(user_options),known_options)
        if (length(unknown)>0) {
            stop("unknown AntBioR options: ", str_c(unknown,collapse=", "))
        }

        current_options=getOption(antbio_option_name)
        if (is.null(current_options)) {
            ## AntBioR options have not been set yet, so set them to the defaults
            current_options=default_options;
            ## set the global option
            temp=list(current_options)
            names(temp)=antbio_option_name
            options(temp)
        }


        ## override any defaults with user-specified options
        if (length(user_options)>0) {
            for (i in 1:length(user_options)) {
                this_option_name=names(user_options)[i]
                if (! is.null(allowed_options[[this_option_name]])) {
                    ## there are restrictions on the allowed values for this option
                    ## could use match.arg here but the output is a bit obscure
                    if (! (user_options[[i]] %in% allowed_options[[this_option_name]])) {
                        stop("value \"",user_options[[i]],"\" is not a valid choice for ",this_option_name," (should be one of ", str_c(allowed_options[[this_option_name]],collapse=", "),")")
                    }
                }
                ## any other specific checks ...
                if (identical(this_option_name,"cache_directory")) {
                    if (!see_if(is.string(user_options[[i]]))) {
                        stop("cache_directory should be a string")
                    }
                    ## strip trailing file separator, if there is one
                    user_options[[i]]=sub("[/\\]+$","",user_options[[i]])
                    if (! (file.exists(user_options[[i]]) && file.info(user_options[[i]])$isdir)) {
                        ## cache directory does not exist. We could create it, but this is probably better left to the user to manage
                        stop("cache directory ",user_options[[i]]," does not exist");
                    }
                }
                if (identical(this_option_name,"user_agent")) {
                    if (!see_if(is.string(user_options[[i]]))) {
                        stop("user_agent should be a string")
                    }
                }
                if (identical(this_option_name,"verbose")) {
                    if (!see_if(is.flag(user_options[[i]]))) {
                        stop("verbose should be TRUE or FALSE")
                    }
                }

                current_options[this_option_name]=user_options[[i]]
            }
            ## set the global option
            temp=list(current_options)
            names(temp)=antbio_option_name
            options(temp)
        } else {
            ## no user options were provided, so user is asking for current options to be returned
            current_options
        }
    }
}


