#' Get diet data from the Southern Ocean dietary database
#'
#' @author Australian Antarctic Data Centre \email{aadc@@aad.gov.au}
#' @references \url{http://www1.data.antarctica.gov.au/aadc/trophic/}
#'
#' @param predator_name string: scientific name of the predator
#' @param prey_name string: scientific name of the prey
#' @param east numeric: easternmost longitude of area of interest (either 0--360 or -180--180, where negative longitudes are western hemisphere)
#' @param west numeric: westernmost longitude of area of interest (either 0--360 or -180--180, where negative longitudes are western hemisphere)
#' @param south numeric: southernmost latitude of area of interest (negative for southern hemisphere latitudes)
#' @param north numeric: northernmost latitude of area of interest (negative for southern hemisphere latitudes)
#'
#' @return A data frame of results
#'
#' @examples
#' get_diet_data(predator_name="Moroteuthis knipovitchi")
#'
#' @export
#'
get_diet_data=function(predator_name,prey_name,north,south,east,west) {
    ##this_url=parse_url("http://localhost:8888")## for testing
    this_url=parse_url(antbio_config()$base_url_aadc)
    this_url$path="aadc/trophic/ws.cfm"
    this_url$query=list()
    if (!missing(predator_name)) {
        assert_that(is.notempty.string(predator_name))
        this_url$query$predator_name=predator_name
    }
    if (!missing(prey_name)) {
        assert_that(is.notempty.string(prey_name))
        this_url$query$prey_name=prey_name
    }
    if (!missing(north)) {
        assert_that(is.numeric(north),north>= -90,north<=90)
        this_url$query$north=north
    }
    if (!missing(south)) {
        assert_that(is.numeric(south),south>= -90,south<=90)
        if (!missing(north)) {
            assert_that(south<north)
        }
        this_url$query$south=south
    }
    if (!missing(east)) {
        assert_that(is.numeric(east),east>= -180,east<=360) ## either 0--360 or -180--180
        this_url$query$east=east
    }
    if (!missing(west)) {
        assert_that(is.numeric(west),west>= -180,west<=360) ## either 0--360 or -180--180
        this_url$query$west=west
    }
    if (length(this_url$query)<1) {
        stop("must provide at least predator_name, prey_name, or spatial constraints")
    }
    this_url$query$format="csv"
    this_url=build_url(this_url)
    x=cached_get(this_url)
    x=str_trim(x)
    x=x[nchar(x)>0]
    x=read.table(header=TRUE,sep=",",stringsAsFactors=FALSE,text=x)
    names(x)=tolower(names(x))
    x
}
